package ch.ethz.ssh2.signature;

import java.io.IOException;

import ch.ethz.ssh2.crypto.SimpleDERReader;
import ch.ethz.ssh2.crypto.digest.SHA1;
import java.math.BigInteger;
import ch.ethz.ssh2.packets.TypesReader;

/**
 * RSASha1Verify.
 * 
 * @author Christian Plattner, plattner@inf.ethz.ch
 * @version $Id: RSASha1Verify.java,v 1.2 2005/06/27 07:33:40 cplattne Exp $
 */
public class RSASha1Verify
{
	static class RSASignature
	{
		private BigInteger s;

		public BigInteger getS()
		{
			return s;
		}

		public RSASignature(BigInteger s)
		{
			this.s = s;
		}
	}

	public static RSAPublicKey parseRSAPublicKey(byte[] key) throws IOException
	{
		TypesReader tr = new TypesReader(key);

		String key_format = tr.readString();

		if (key_format.equals("ssh-rsa") == false)
			throw new IllegalArgumentException("This is not a ssh-rsa public host key");

		BigInteger e = tr.readMPINT();
		BigInteger n = tr.readMPINT();

		if (tr.remain() != 0)
			throw new IOException("Padding in RSA public key!");

		return new RSAPublicKey(e, n);
	}

	public static RSASignature parseRSASignature(byte[] sig) throws IOException
	{
		TypesReader tr = new TypesReader(sig);

		String sig_format = tr.readString();

		if (sig_format.equals("ssh-rsa") == false)
			throw new IOException("Peer sent wrong signature format");

		byte[] rsArray = tr.readByteString();

		if (tr.remain() != 0)
			throw new IOException("Padding in RSA signature!");

		/*
		 * Work around the sign issue by allocating a leading zero byte.
		 */

		byte[] tmp = new byte[rsArray.length + 1];

		System.arraycopy(rsArray, 0, tmp, 1, rsArray.length);
		BigInteger s = new BigInteger(tmp);

		return new RSASignature(s);
	}

	public static boolean verifySignature(byte[] message, byte[] sshRsaSig, byte[] sshRsaHostKey) throws IOException
	{
		RSAPublicKey dpk = parseRSAPublicKey(sshRsaHostKey);
		RSASignature ds = parseRSASignature(sshRsaSig);

		SHA1 md = new SHA1();
		md.update(message);
		byte[] sha_message = new byte[md.getDigestLength()];
		md.digest(sha_message);

		BigInteger n = dpk.getN();
		BigInteger e = dpk.getE();

		BigInteger s = ds.getS();

		if (BigInteger.ZERO.compareTo(s) >= 0 || n.compareTo(s) <= 0)
		{
			return false;
		}

		/* RSA, PKCS 1 v1.5 padding, DER, ASN.1... */

		byte[] v = s.modPow(e, n).toByteArray();

		if (v.length <= 10)
			return false;

		int pos = 0;

		if (v[0] == 0)
			pos++;

		if ((v[pos] != 1) && (v[pos] != 2))
			return false;

		pos++;

		while ((pos < v.length) && (v[pos] != 0))
			pos++;

		pos++;

		if ((pos < 10) || (pos >= v.length))
			return false;

		SimpleDERReader dr = new SimpleDERReader(v, pos, v.length - pos);

		byte[] seq = dr.readSequenceAsByteArray();
		dr.resetInput(seq);
			
		/* Read and ignore digestAlgorithm */
		dr.readSequenceAsByteArray();

		byte[] digest = dr.readOctetString();
		
		if (digest.length != sha_message.length)
			return false;

		for (int i = 0; i < sha_message.length; i++)
		{
			if (sha_message[i] != digest[i])
				return false;
		}

		return true;
	}
}
